/*
    Based on DefinePlugin by Tobias Koppers @sokra
    MIT License http://www.opensource.org/licenses/mit-license.php
*/
var ConstDependency = require("webpack/lib/dependencies/ConstDependency");
var BasicEvaluatedExpression = require("webpack/lib/BasicEvaluatedExpression");

var NullFactory = require("webpack/lib/NullFactory");

function InlineConstantsPlugin(definitions) {
    this.definitions = definitions;
}

module.exports = InlineConstantsPlugin;

InlineConstantsPlugin.prototype.apply = function(compiler) {
    compiler.plugin("compilation", function(compilation) {
        compilation.dependencyFactories.set(ConstDependency, new NullFactory());
        compilation.dependencyTemplates.set(ConstDependency, new ConstDependency.Template());
    });
    (function walkDefinitions(definitions, prefix) {
        Object.keys(definitions).forEach(function(key) {
            var code = definitions[key];
            if(code && typeof code === "object" && !(code instanceof RegExp)) {
                walkDefinitions(code, prefix + key + ".");
                return;
            }
            applyDefineKey(prefix, key);
            applyDefine(prefix + key, code);
        });
    }(this.definitions, ""));

    function toCode(code) {
        if(code === null) return "null";
        else if(code === undefined) return "undefined";
        else if(code instanceof RegExp && code.toString) return code.toString();
        else if(typeof code === "function" && code.toString) return code.toString();
        else if(typeof code === "object") return code;
        else return code + "";
    }

    function applyDefineKey(prefix, key) {
        var splittedKey = key.split(".");
        splittedKey.slice(1).forEach(function(_, i) {
            var fullKey = prefix + splittedKey.slice(0, i + 1).join(".");
            compiler.parser.plugin("can-rename " + fullKey, function() {
                return true;
            });
        });
    }

    function applyDefine(key, code) {
        var recurse = false;
        var recurseTypeof = false;
        code = toCode(code);

        compiler.parser.plugin("can-rename " + key, function() {
            return true;
        });
        compiler.parser.plugin("evaluate Identifier " + key, function(expr) {
            if(recurse) return;
            recurse = true;
            var res = compiler.parser.evaluate(code);
            recurse = false;
            res.setRange(expr.range);
            return res;
        });
        compiler.parser.plugin("expression " + key, function(expr) {
            var dep = new ConstDependency("(" + code + ")", expr.range);
            dep.loc = expr.loc;
            this.state.current.addDependency(dep);
            return true;
        });
    }
};